@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 
-/* Copyright (C) 1998 Redhat Software Inc. 
+/* Copyright (C) 1998 Redhat Software Inc.
  * Authors: Jonathan Blandford <jrb@redhat.com>
  */
 #include "tree.h"
@@ -21,6 +21,8 @@ GdkBitmap *mask3;*/
 
 extern gchar *init_cap;
 extern GnomeAppBar *status_bar;
+static void selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column, gpointer unseen);
+static void unselected_row_callback (GtkCList *clist, gint row, gint column, GdkEvent *event);
 
 #define SMALL_ICON_SIZE 20
 #if 0
@@ -44,7 +46,8 @@ init_tree_handler (GtkWidget *ctree, GdkEvent *event, node_data *data)
                 gnome_appbar_push (GNOME_APPBAR (status_bar), (data->gde->name));
         launch_capplet (data, TRUE);
 }
-gint compare_last_dir (gchar *first, gchar *second)
+static gint
+compare_last_dir (gchar *first, gchar *second)
 {
         gboolean retval;
         gchar *temp1;
@@ -60,7 +63,7 @@ gint compare_last_dir (gchar *first, gchar *second)
         g_free (temp2);
         return retval;
 }
-gboolean
+static gboolean
 compare_nodes (GnomeDesktopEntry *data1, GnomeDesktopEntry *data2)
 {
         g_return_val_if_fail (data1, FALSE);
@@ -68,7 +71,7 @@ compare_nodes (GnomeDesktopEntry *data1, GnomeDesktopEntry *data2)
         g_return_val_if_fail (data1->type, FALSE);
         g_return_val_if_fail (data2->type, FALSE);
 
-        if (!strcmp (data1->type, "Directory") && 
+        if (!strcmp (data1->type, "Directory") &&
             (!strcmp (data2->type, "Directory")))
                 return compare_last_dir (data1->location,
                                          data2->location);
@@ -76,7 +79,7 @@ compare_nodes (GnomeDesktopEntry *data1, GnomeDesktopEntry *data2)
                 return (!strcmp (rindex (data1->location,'/'),
                                  rindex (data2->location,'/')));
 }
-/* 
+/*
  * This function is used to generate a node starting at a directory.
  * It doesn't do all that complex error checking -- if something
  * happens, it just returns null and skips the directory.
@@ -96,7 +99,7 @@ merge_nodes (GNode *node1, GNode *node2)
         /* first we merge data */
         if (node1->data == NULL)
                 node1->data = node2->data;
-        else if (node2->data != NULL) 
+        else if (node2->data != NULL)
                 gnome_desktop_entry_free (node2->data);
 
         /* now we want to find subdirs to merge */
@@ -134,12 +137,12 @@ read_directory (gchar *directory)
 
         while ((child_dir = readdir (parent_dir)) != NULL) {
                 if (child_dir->d_name[0] != '.') {
-                                        
+
                         /* we check to see if it is interesting. */
                         GString *name = g_string_new (directory);
                         g_string_append (name, "/");
                         g_string_append (name, child_dir->d_name);
-                        
+
                         if (stat (name->str, &filedata) != -1) {
                                 gchar* test;
                                 if (S_ISDIR (filedata.st_mode)) {
@@ -170,7 +173,7 @@ read_directory (gchar *directory)
                 /* FIXME: i guess we should free memory now... */
                 return NULL;
         }
-                
+
         if (retval->children == NULL) {
                 if (retval->data)
                         gnome_desktop_entry_free (retval->data);
@@ -178,7 +181,8 @@ read_directory (gchar *directory)
         }
         return retval;
 }
-void
+
+static void
 generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
 {
         GNode *i;
@@ -190,7 +194,7 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
         gchar *icon;
         GnomePixmap *icon_gpixmap;
         GdkPixmap *icon_pixmap, *icon_mask;
-        
+
 
         text[1] = NULL;
         for (i = node;i;i = i->next) {
@@ -205,7 +209,7 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
 #if 0
                 if ((i->data) && ((GnomeDesktopEntry *)i->data)->icon)
                         icon = ((GnomeDesktopEntry *)i->data)->icon;
-                if (icon && g_file_exists (icon)) 
+                if (icon && g_file_exists (icon))
                         icon_gpixmap = (GnomePixmap *)gnome_pixmap_new_from_file_at_size(icon,
                                                                                          SMALL_ICON_SIZE,
                                                                                          SMALL_ICON_SIZE);
@@ -215,11 +219,11 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
                 }
 #endif
                 if (i->data && (!strcmp(((GnomeDesktopEntry *)i->data)->type,"Directory")))
-                        child = gtk_ctree_insert_node (ctree,parent,NULL, text, 3, 
+                        child = gtk_ctree_insert_node (ctree,parent,NULL, text, 3,
                                                        icon_pixmap, icon_mask,
                                                        icon_pixmap, icon_mask, FALSE,FALSE);
-                else 
-                        child = gtk_ctree_insert_node (ctree,parent,NULL, text, 3,  
+                else
+                        child = gtk_ctree_insert_node (ctree,parent,NULL, text, 3,
                                                        icon_pixmap, icon_mask,
                                                        icon_pixmap, icon_mask, TRUE,FALSE);
                 data = g_malloc (sizeof (node_data));
@@ -238,13 +242,14 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
                         for (j = 0; data->gde->exec[j]; j++)
                                 strncat (exec_collapsed, data->gde->exec[j], 99);
                         if (!strcmp (exec_collapsed, init_cap)) {
-                                GtkCTreeNode *temp = child;
 /*
+                                GtkCTreeNode *temp = child;
+
                                 for (;temp;temp = GTK_CTREE_ROW (temp)->parent)
-                                        gtk_ctree_expand (ctree, temp);
+                                gtk_ctree_expand (ctree, temp);
 */
                                 gtk_ctree_select (ctree, child);
-                                
+
                                 gtk_signal_connect_after (GTK_OBJECT (ctree), "expose_event",
                                                           GTK_SIGNAL_FUNC (init_tree_handler),
                                                           data);
@@ -257,6 +262,8 @@ generate_tree_helper (GtkCTree *ctree, GtkCTreeNode *parent, GNode *node)
                         gtk_ctree_expand_recursive (ctree, child);
         }
 }
+
+
 GtkWidget *
 generate_tree ()
 {
@@ -271,9 +278,9 @@ generate_tree ()
         /* I don't remember what this was for, but away it goes... -jrb */
         gtk_signal_connect (GTK_OBJECT (retval), "button_press_event",
                             GTK_SIGNAL_FUNC (button_press), NULL);
-#endif   
+#endif
         /* First thing we want to do is to check directories to create the menus */
-        
+
         gtk_clist_set_row_height(GTK_CLIST (retval),20);
         gtk_ctree_set_line_style (GTK_CTREE (retval), GTK_CTREE_LINES_DOTTED);
         gtk_ctree_set_expander_style (GTK_CTREE (retval), GTK_CTREE_EXPANDER_SQUARE);
@@ -282,23 +289,24 @@ generate_tree ()
         gtk_ctree_set_indent (GTK_CTREE (retval), 15);
         gtk_clist_set_column_auto_resize (GTK_CLIST (retval), 0, TRUE);
         /*gtk_widget_set_usize (retval, 200, 375);*/
-        gtk_clist_set_selection_mode(GTK_CLIST(retval), GTK_SELECTION_SINGLE);
+        gtk_clist_set_selection_mode(GTK_CLIST(retval), GTK_SELECTION_SINGLE );
         gtk_signal_connect( GTK_OBJECT (retval),"tree_select_row", GTK_SIGNAL_FUNC (selected_row_callback), NULL);
+        gtk_signal_connect( GTK_OBJECT (retval),"unselect_row", GTK_SIGNAL_FUNC (unselected_row_callback), NULL);
 
         /* load the directories */
         root_prefix = gnome_unconditional_datadir_file ("control-center");
         global_node = read_directory (root_prefix);
         user_prefix = gnome_util_home_file ("control-center");
         user_node = read_directory (user_prefix);
-        
+
         if (user_node == NULL)
                 user_node = global_node;
-        else 
+        else
                 merge_nodes (user_node,global_node);
 
         /* now we actually set up the tree... */
-        /* we prolly want to use the gtree_insert_node function to do this, 
-         * but as it was written after the code here... 
+        /* we prolly want to use the gtree_insert_node function to do this,
+         * but as it was written after the code here...
          *
          * we do user_node->children to avoid the root menu.
          */
@@ -316,8 +324,21 @@ generate_tree ()
         return retval;
 }
 
-void
-selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column, gpointer data)
+
+static void
+unselected_row_callback (GtkCList       *clist,
+                         gint            row,
+                         gint            column,
+                         GdkEvent       *event)
+{
+
+        if (event != NULL) {
+                gtk_clist_select_row (clist, row, column);
+        }
+}
+
+static void
+selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column, gpointer userdata)
 {
         node_data *data;
         GnomeDesktopEntry *gde;
@@ -327,7 +348,7 @@ selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column, gpoin
         if (data == NULL)
                 return;
 
-        gde = (GnomeDesktopEntry *)data->gde; 
+        gde = (GnomeDesktopEntry *)data->gde;
         gnome_appbar_pop (GNOME_APPBAR (status_bar));
         if (gde->comment)
                 gnome_appbar_push (GNOME_APPBAR (status_bar), (gde->comment));
@@ -345,7 +366,7 @@ selected_row_callback (GtkWidget *widget, GtkCTreeNode *node, gint column, gpoin
                 if (gdk_pointer_is_grabbed ())
                         gdk_pointer_ungrab (GDK_CURRENT_TIME);
                 gdk_flush ();
-                
+
                 launch_capplet (data, TRUE);
         }
 }
